// This file is part of Applet Manager.
//
// Applet Manager is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Applet Manager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Applet Manager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "appletmgr.h"
#include "appletmanager.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>
#include <dirent.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmessagebox.h>

#define VERSION "v1.0"

static const char* const iEnabled_data[] = {
"12 12 7 1",
". c None",
"# c #006500",
"d c #007100",
"a c #007900",
"e c #008500",
"b c #008d00",
"c c #00ce00",
"....####....",
"..########..",
".###abba###.",
".##bccccb##.",
"##accccccd##",
"##bcccccce##",
"##eccccccb##",
"##dccccccd##",
".##bcccce##.",
".###dbbd###.",
"..########..",
"....####...."};

static const char* const iDisabled_data[] = {
"12 12 2 1",
". c None",
"# c #cd0000",
"....####....",
"..########..",
".##########.",
".####...###.",
"######...###",
"###.###..###",
"###..###.###",
"###...######",
".###...####.",
".##########.",
"..#######...",
"....####...."};

QPixmap *iEnabled, *iDisabled;
QString configFile;
int enableDirty, disableDirty, restartFlag;

AppletManager::AppletManager( QWidget* parent,  const char* name, WFlags fl )
    : AppletMgrBase( parent, name, fl )
{
    
    Config *cfgFile;
    QString homeDir, cDirTemp, tTitle;

    tTitle = "Applet Manager ";
    tTitle.append( VERSION );

    setCaption( tTitle );

    homeDir = getenv("HOME");

    if ( homeDir.right(1) != "/" ) {
        homeDir.append("/");
    }
    configFile = homeDir;
    configFile.append("Settings/appletmanager.conf");
   
    iEnabled = new QPixmap( ( const char** )iEnabled_data );
    iDisabled = new QPixmap( ( const char** )iDisabled_data );
    
    enableDirty = disableDirty = restartFlag = 0;
    
    connect( (QObject *)cmdDisableSel, SIGNAL(clicked()), this, SLOT(cmdDisableSel_clicked()) );
    connect( (QObject *)cmdEnableSel, SIGNAL(clicked()), this, SLOT(cmdEnableSel_clicked()) );    
    connect( (QObject *)cmdSetPath, SIGNAL(clicked()), this, SLOT(cmdSetPath_clicked()) );

    cfgFile = new Config( configFile, Config::File );
    cfgFile->setGroup( "Main" );
    cDirTemp = cfgFile->readEntry( "lastDir", "/opt/QtPalmtop/plugins/applets" );
    txtInputPath->setText( cDirTemp );
    delete cfgFile;

    cmdSetPath_clicked();

}

void AppletManager::closeEvent( QCloseEvent *e ) {

    if (enableDirty == 1 || disableDirty == 1 ) {
        int returnVal;
        QString sMsg;
        sMsg = "You have changed your applet\nloading preferences. Do you want to\nrestart the window manager now?";
        QMessageBox msgBox( QString( "Applet Manager" ), sMsg, QMessageBox::Critical, QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::NoButton, NULL);
        returnVal = msgBox.exec();
        if (returnVal == 3) {
	    restartFlag = 1;
        }
    }


    e->accept();

}

AppletManager::~AppletManager()
{
    if (restartFlag == 1) {
        QCopEnvelope e( "QPE/System", "restart()" );
    }
}

void AppletManager::cmdEnableSel_clicked()
{

    for (int i = 0; i < tblAppletList->numRows(); i++) {
        if ( tblAppletList->isSelected( i ) ) {
	    if ( tblAppletList->pixmap( i )->serialNumber() == iEnabled->serialNumber() ) {
	    } else {
	        
		QString newFilename;
		QString oldFilename;
		
		newFilename = readPath;
		if (newFilename.right(1) != "/") {
		    newFilename.append("/");
		}
		newFilename.append( tblAppletList->text( i ) );

		oldFilename = readPath;
		if (oldFilename.right(1) != "/") {
		    oldFilename.append("/");
		}
		oldFilename.append(".");
		oldFilename.append( tblAppletList->text( i ) );

		rename( oldFilename, newFilename );
                
		enableDirty = 1;

	    }
	}
    }

    cmdSetPath_clicked();

}

void AppletManager::cmdDisableSel_clicked()
{

    for (int i = 0; i < tblAppletList->numRows(); i++) {
        if ( tblAppletList->isSelected( i ) ) {
            if ( tblAppletList->pixmap( i )->serialNumber() == iDisabled->serialNumber() ) {
            } else {

                QString newFilename;
                QString oldFilename;

                newFilename = readPath;
                if (newFilename.right(1) != "/") {
                    newFilename.append("/");
                }
		newFilename.append(".");
                newFilename.append( tblAppletList->text( i ) );

                oldFilename = readPath;
                if (oldFilename.right(1) != "/") {
                    oldFilename.append("/");
                }
                oldFilename.append( tblAppletList->text( i ) );

                rename( oldFilename, newFilename );

		disableDirty = 1;

            }
        }
    }
    
    cmdSetPath_clicked();

}

void AppletManager::cmdSetPath_clicked()
{

    struct stat checkDir;
    DIR* curDir;
    dirent *curFile = 0;
    QString dirString;
    QStringList fileList;

    readPath = txtInputPath->text();
    tblAppletList->clear();

    if (stat( (const char *)readPath, &checkDir ) == -1 ) {
        QString sMsg;
	sMsg = "Could not find the directory you\nspecified.";
        QMessageBox msgBox( QString( "Applet Manager" ), sMsg, QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton );
	msgBox.exec();
	txtInputPath->setText( lastGood );
	cmdSetPath_clicked();
    } else {
        Config *cfgFile;
	cfgFile = new Config( configFile, Config::File );
	cfgFile->setGroup( "Main" );
	cfgFile->writeEntry( "lastDir", readPath );
	lastGood = readPath;
	delete cfgFile;
        curDir = opendir( (const char *)readPath );
        while ( (curFile = readdir(curDir)) ) {
            dirString = curFile->d_name;
	    if (dirString.right(3) == ".so" && dirString != "." && dirString != "..") {
	        if (dirString.left(1) == "." ) {
		    tblAppletList->insertItem( *iDisabled, dirString.right(dirString.length()-1) );
		} else {
		    tblAppletList->insertItem( *iEnabled, dirString );
		}
	    }
        }
        closedir(curDir);

	tblAppletList->sort();
        
    }

}

